/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus;

import java.io.IOException;
import java.util.List;

// Zeus imports
import org.enhydra.zeus.ZeusException;
import org.enhydra.zeus.transform.NoSuchValueEnumerationException;
import org.enhydra.zeus.transform.TransformerOptions;
import org.enhydra.zeus.transform.ValueEnumeration;

/**
 * <p>
 *  <code>Transformer</code> is the interface for converting a set of Zeus
 *    <code>{@link Binder}</code> objects from a set of XML-based names 
 *    (created by a Zeus <code>{@link Binder}</code> instance) and
 *    identities to another set of <code>Binding</code> objects. This second
 *    set of objects is only an intermediate set, which is then used by a
 *    <code>{@link Generator}</code> to create Java classes.
 * </p><p>
 *  This allows a user to specify a <i>mapping file</i> and thereby transform
 *    a set of XML names into another set of names, either for ensuring legal
 *    Java names, for using more business-centric names, or any other user
 *    desired basis.
 * </p>
 *
 * @author Brett McLaughlin
 */
public interface Transformer {

    /**
     * <p>
     *  This allows setting of the <code>{@link TransformerOptions}</code> for
     *    this transformer.
     * </p>
     *
     * @param transformerOptions the options for this transformer
     */
    public void setTransformerOptions(TransformerOptions transformerOptions);
    
    /**
     * <p>
     *  This returns the current <code>{@link TransformerOptions}</code> for
     *    this transformer.
     * </p>
     *
     * @return <code>TransformerOptions</code> - the options in use for this
     *         transformer.
     */
    public TransformerOptions getTransformerOptions();
    
    /**
     * <p>
     *  This will add a new <code>{@link ValueEnumeration}</code> to those
     *    available to this <code>Transformer</code>.
     * </p>
     *
     * @param valueEnumeration the enumeration to add for this transformer.
     */
    public void addValueEnumeration(ValueEnumeration valueEnumeration);
    
    /**
     * <p>
     *  This will retrieve the <code>{@link ValueEnumeration}</code> for the
     *    supplied name.
     * </p><p>
     *  If the supplied name is not associated with a 
     *    <code>ValueEnumeration<code>, this is considered an <i>exceptional</i>
     *    condition; any <code>Transformer</code> construct looking for a
     *    <code>ValueEnumeration</code> by name <i>expects</i> to find one,
     *    or the supplied mapping information is in error. Therefore, 
     *    in this case, a <code>{@link NoSuchValueEnumerationException}</code>
     *    is thrown.
     * </p>
     *
     * @return <code>ValueEnumeration</code> - the enumeration for the supplied
     *         name.
     * @throws <code>NoSuchValueEnumerationException</code> - when an
     *         enumeration with the supplied name cannot be found.
     */
    public ValueEnumeration getValueEnumeration(String name) 
        throws NoSuchValueEnumerationException;

    /**
     * <p>
     *  This method performs the work of transforming a set of Zeus
     *    <code>{@link Binding}</code> objects. It then returns a <i>new</i>
     *    set of <code>Binding</code> objects (in <code>List</code> form)
     *    with transformed names, types, and values.
     * </p><p>
     *  When this method is invoked, it attempts to see if parsing of the 
     *    mapping file has already occurred. If it has not, parsing of that
     *    file will occur within this method.
     * </p><p>
     *  It is important to note that the <i>number</i> of bindings returned
     *    from this method may not be the same as the number supplied. This
     *    is due to some XML elements/attributes being converted to value
     *    objects, while other mapping instructions may actually create new
     *    binding objects.
     * </p>
     *
     * @param bindings <code>List</code> of bindings to process for
     *        transformation.
     * @return <code>List</code> - the transformed bindings.
     * @throws <code>IOException</code> - when errors occur in parsing
     *         a mapping configuration file.
     * @throws <code>ZeusException</code> - when errors occuring in
     *         transformation.
     */
    public List transform(List bindings) throws IOException, ZeusException;
}
