/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.util;

import java.io.IOException;
import java.util.Iterator;
import java.util.List;

// Zeus imports
import org.enhydra.zeus.Binder;
import org.enhydra.zeus.Binding;
import org.enhydra.zeus.Generator;
import org.enhydra.zeus.ZeusException;
import org.enhydra.zeus.binder.DTDBinder;
import org.enhydra.zeus.generator.SimpleGenerator;
import org.enhydra.zeus.source.DTDSource;
import org.enhydra.zeus.source.StreamDTDSource;

/** 
 * <p>
 *  This is a standalone "utility" class. It allows source code
 *    generation from a DTD.
 * </p>
 *
 * @author Brett McLaughlin
 */
public class DTDSourceGenerator extends BaseSourceGenerator
    implements SourceGenerator {

    /** The interface package name to use. */
    private static String interfacePackageName;

    /**
     * <p>
     *  This sets up defaults for the generator.
     * </p>
     */
    public DTDSourceGenerator() {
        super();
    }

    /**
     * <p>
     *  This defines a contract for subclasses to allow them to get the
     *    Zeus <code>{@link org.enhydra.zeus.Binding}</code> objects to
     *    generate code from.
     * </p>
     *
     * @return <code>List</code> - a list of the Binding
     *         objects to generate code from.
     * @exception <code>IOException</code> - when binding creation fails
     */
    protected List getConstraintBindings() throws IOException {

        // Create a source to read the DTD from.
        DTDSource dtdSource = new StreamDTDSource(constraintsReader);

        // Create the binder we will use.
        DTDBinder dtdBinder  = new DTDBinder(dtdSource);
        dtdBinder.setIsCollapsingSimpleElements(collapseSimpleElements);
        dtdBinder.setIsIgnoringIDAttributes(ignoreIDAttributes);

        // return the bindings.
        return dtdBinder.getBindings();
    }

    /**
     * <p>
     *  This handles the process of creating a <code>{@link Generator}</code>
     *    for use in data binding. 
     * </p><p>
     *  This version returns determines the <code>Generator</code> 
     *    implementation to use based on user input.
     * </p>
     *
     * @return <code>Generator</code> - the <code>Generator</code> to use.
     */
    protected Generator getGenerator() {
        Generator generator = new SimpleGenerator();
        return generator;
    }       

    /**
     * <p>
     *  This provides command-line class generation.
     * </p>
     *
     * @param args <code>String</code> array of command-line arguments.
     */
    public static void main(String[] args) {
        try {
            Arguments arguments = new Arguments(args);

            // Generate source
            DTDSourceGenerator sourceGenerator =
                new DTDSourceGenerator();

            // Ensure correct usage
            String constraintsFilename = 
                arguments.getValue("constraints");
            if (constraintsFilename == null) {
                System.out.println("Usage: java " +
                                   sourceGenerator.getClass().getName() +
                                   " -constraints=<constraints filename>" +
                                   " [-outputDir=<output directory>]" +
                                   " [-collapseSimpleElements=" +
                                       "<true | false>]" +
                                   " [-ignoreIDAttributes=<true | false>]" +
                                   " [-javaPackage=<Java package name>]"); 
                return;
            }

            // Get command-line options
            String outputDirName = arguments.getValue("outputDir");
            String collapseSimpleElements = 
                arguments.getValue("collapseSimpleElements");
            String ignoreIDAttributes = 
                arguments.getValue("ignoreIDAttributes");
            String javaPackage = arguments.getValue("javaPackage");

            sourceGenerator.setConstraintsInput(constraintsFilename);

            if (outputDirName != null) {
                sourceGenerator.setOutputDir(outputDirName);
            }
            if (javaPackage != null) {
                sourceGenerator.setJavaPackage(javaPackage);
            }
            if ((collapseSimpleElements != null) &&
                (collapseSimpleElements.equalsIgnoreCase("true"))) {
                if ((ignoreIDAttributes != null) &&
                    (ignoreIDAttributes.equalsIgnoreCase("true"))) {
                    sourceGenerator.setCollapseSimpleElements(true, true);
                } else {
                    sourceGenerator.setCollapseSimpleElements(true, false);
                }
            }

            // Perform actual class generation
            sourceGenerator.generate();

        } catch (IOException e) {
            System.out.println("Error generating source code: " +
                               e.getMessage());
            e.printStackTrace();
        } catch (ZeusException e) {
            System.out.println("Error generating source code: " +
                               e.getMessage());
            e.printStackTrace();
        }
    }
}
