/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

// Zeus imports
import org.enhydra.zeus.InvalidCollectionTypeException;

/**
 * <p>
 *  <code>AbstractBindingTest</code> is an abstract test case for 
 *    <code>Binding</code> objects.  This class provides tests for
 *    methods on the <code>org.enhydra.zeus.Binding</code> interface that 
 *    all <code>Binding</code> subclasses will inherit.
 * </p><p>
 *  Concrete <code>Binding</code> test cases should extend this class, 
 *    override the <code>{@link createBinding}</code> method, and add any 
 *    tests specific to the class being tested.
 * </p>
 *
 * @author Robert Sese
 */
public abstract class AbstractBindingTest extends TestCase {

    /**
     * <p>
     *  This creates a new <code>Binding</code> object that will be used to 
     *    test <code>Binding</code> methods.
     * </p>
     *
     * @return <code>Binding</code> - the created <code>Binding</code>
     */
    public abstract Binding createBinding();

    /**
     * <p>
     *  Constructor
     * </p>
     *
     * @param name the name of the test for use in reports
     */
    public AbstractBindingTest(String name) {
        super(name);
    }
    
    /**
     * <p>
     *  This method maeks it easier to call these
     *    tests dynamically.
     * </p>
     *
     * @return <code>TestSuite</code> - corresponds to this 
     *         <code>TestCase</code>.
     */
    public static Test suite() {
        return new TestSuite(AbstractBindingTest.class);
    }
    
    /**
     * <p>
     *  This tests the <code>getXMLName()</code> method.
     * </p>
     */
    public void testGetXMLName() {
        Binding binding = createBinding();
        String expected = "my_xml_name";

        binding.setXMLName(expected);
        assertEquals(expected, binding.getXMLName());
    }

    /**
     * <p>
     *  This tests the <code>setXMLName()</code> method.
     * </p>
     */
    public void testSetXMLName() {
        Binding binding = createBinding();
        String expected1 = "my_xml_name_1";
        String expected2 = "my_xml_name_2";

        binding.setXMLName(expected1);
        assertEquals(expected1, binding.getXMLName());
        binding.setXMLName(expected2);
        assertEquals(expected2, binding.getXMLName());
    }
    
    /**
     * <p>
     *  This tests the <code>getXMLNamespaceURI()</code> method.
     * </p>
     */
    public void testGetXMLNamespaceURI() {
        Binding binding = createBinding();
        String expected = "http://www.namespace.com";

        binding.setXMLNamespaceURI(expected);
        assertEquals(expected, binding.getXMLNamespaceURI());
    }

    /**
     * <p>
     *  This tests the <code>setXMLNamespaceURI()</code> method.
     * </p>
     */
    public void testSetXMLNamespaceURI() {
        Binding binding = createBinding();
        String expected1 = "http://www.namespace1.com";
        String expected2 = "http://www.namespace2.com";

        binding.setXMLNamespaceURI(expected1);
        assertEquals(expected1, binding.getXMLNamespaceURI());
        binding.setXMLNamespaceURI(expected2);
        assertEquals(expected2, binding.getXMLNamespaceURI());
    }
    
    /**
     * <p>
     *  This tests the <code>getXMLType()</code> method.
     * </p>
     */
    public void testGetXMLType() {
        Binding binding = createBinding();
        String expected = "my_xml_type";

        binding.setXMLType(expected);
        assertEquals(expected, binding.getXMLType());
    }

    /**
     * <p>
     *  This tests the <code>setXMLType()</code> method.
     * </p>
     */
    public void testSetXMLType() {
        Binding binding = createBinding();
        String expected1 = "my_xml_type_1";
        String expected2 = "my_xml_type_2";

        binding.setXMLType(expected1);
        assertEquals(expected1, binding.getXMLType());
        binding.setXMLType(expected2);
        assertEquals(expected2, binding.getXMLType());
    }
    
    /**
     * <p>
     *  This tests the <code>getXMLTypeNamespaceURI()</code> method.
     * </p>
     */
    public void testGetXMLTypeNamespaceURI() {
        Binding binding = createBinding();
        String expected = "http://www.typeNamespace.com";

        binding.setXMLTypeNamespaceURI(expected);
        assertEquals(expected, binding.getXMLTypeNamespaceURI());
    }

    /**
     * <p>
     *  This tests the <code>setXMLTypeNamespaceURI()</code> method.
     * </p>
     */
    public void testSetXMLTypeNamespaceURI() {
        Binding binding = createBinding();
        String expected1 = "http://www.typeNamespace1.com";
        String expected2 = "http://www.typeNamespace2.com";

        binding.setXMLTypeNamespaceURI(expected1);
        assertEquals(expected1, binding.getXMLTypeNamespaceURI());
        binding.setXMLTypeNamespaceURI(expected2);
        assertEquals(expected2, binding.getXMLTypeNamespaceURI());
    }

    /**
     * <p>
     *  This tests the <code>getJavaName()</code> method.
     * </p>
     */
    public void testGetJavaName() {
        Binding binding = createBinding();
        String expected = "myJavaName";

        binding.setJavaName(expected);
        assertEquals(expected, binding.getJavaName());
    }

    /**
     * <p>
     *  This tests the <code>setJavaName()</code> method.
     * </p>
     */
    public void testSetJavaName() {
        Binding binding = createBinding();
        String expected1 = "myJavaName1";
        String expected2 = "myJavaName2";

        binding.setJavaName(expected1);
        assertEquals(expected1, binding.getJavaName());
        binding.setJavaName(expected2);
        assertEquals(expected2, binding.getJavaName());
    }
    
    /**
     * <p>
     *  This tests the <code>getJavaVariableName()</code> method.
     * </p>
     */
    public void testGetJavaVariableName() {
        Binding binding = createBinding();
        String expected = "myJavaVariableName";

        binding.setJavaVariableName(expected);
        assertEquals(expected, binding.getJavaVariableName());
    }

    /**
     * <p>
     *  This tests the <code>setJavaVariableName()</code> method.
     * </p>
     */
    public void testSetJavaVariableName() {
        Binding binding = createBinding();
        String expected1 = "myJavaVariableName1";
        String expected2 = "myJavaVariableName2";

        binding.setJavaVariableName(expected1);
        assertEquals(expected1, binding.getJavaVariableName());
        binding.setJavaVariableName(expected2);
        assertEquals(expected2, binding.getJavaVariableName());
    }

    /**
     * <p>
     *  This tests the <code>getJavaType()</code> method.
     * </p>
     */
    public void testGetJavaType() {
        Binding binding = createBinding();
        String expected = "MyType";

        binding.setJavaType(expected);
        assertEquals(expected, binding.getJavaType());
    }

    /**
     * <p>
     *  This tests the <code>setJavaType()</code> method.
     * </p>
     */
    public void testSetJavaType() {
        Binding binding = createBinding();
        String expected1 = "MyType1";
        String expected2 = "MyType2";

        binding.setJavaType(expected1);
        assertEquals(expected1, binding.getJavaType());
        binding.setJavaType(expected2);
        assertEquals(expected2, binding.getJavaType());
    }

    /**
     * <p>
     *  This tests the <code>getJavaInterfacePackage()</code> method.
     * </p>
     */
    public void testGetJavaInterfacePackage() {
        Binding binding = createBinding();
        String expected = "org.my.interface.package";

        binding.setJavaInterfacePackage(expected);
        assertEquals(expected, binding.getJavaInterfacePackage());
    }

    /**
     * <p>
     *  This tests the <code>setJavaInterfacePackage()</code> method.
     * </p>
     */
    public void testSetJavaInterfacePackage() {
        Binding binding = createBinding();
        String expected1 = "org.my.interface.package1";
        String expected2 = "org.my.interface.package2";

        binding.setJavaInterfacePackage(expected1);
        assertEquals(expected1, binding.getJavaInterfacePackage());
        binding.setJavaInterfacePackage(expected2);
        assertEquals(expected2, binding.getJavaInterfacePackage());
    }
    
    /**
     * <p>
     *  This tests the <code>getJavaImplementationPackage()</code> method.
     * </p>
     */
    public void testGetJavaImplementationPackage() {
        Binding binding = createBinding();
        String expected = "org.my.implementation.package";

        binding.setJavaImplementationPackage(expected);
        assertEquals(expected, binding.getJavaImplementationPackage());
    }

    /**
     * <p>
     *  This tests the <code>setJavaImplementationPackage()</code> method.
     * </p>
     */
    public void testSetJavaImplementationPackage() {
        Binding binding = createBinding();
        String expected1 = "org.my.implementation.package1";
        String expected2 = "org.my.implementation.package2";

        binding.setJavaImplementationPackage(expected1);
        assertEquals(expected1, binding.getJavaImplementationPackage());
        binding.setJavaImplementationPackage(expected2);
        assertEquals(expected2, binding.getJavaImplementationPackage());
    }
    
    /**
     * <p>
     *  This tests the <code>getJavaCollectionClass()</code> method.
     * </p>
     */
    public void testGetJavaCollectionClass() {
        Binding binding = createBinding();
        String expected = "java.util.List";

        try {
            binding.setJavaCollectionClass(expected);
            assertEquals(expected, binding.getJavaCollectionClass());
        } catch (InvalidCollectionTypeException e) {
            // If this failed, the test failed
            assertTrue(false);
        }
    }

    /**
     * <p>
     *  This tests the <code>setJavaCollectionClass()</code> method.
     * </p>
     */
    public void testSetJavaCollectionClass() {
        Binding binding = createBinding();
        String collection1 = "java.util.List";
        String collection2 = "java.util.ArrayList";
        boolean success = true;
        
        try {
            binding.setJavaCollectionClass(collection1);            
            binding.setJavaCollectionClass(collection2);
        } catch (InvalidCollectionTypeException e) {
            // If this failed, the test failed
            success = false;
        }
        
        assertTrue(success);
    }
    
    /**
     * <p>
     *  This tests the <code>getJavaCollectionClass()</code> method, and ensures
     *    that an <code>{@link InvalidCollectionTypeException}</code> is thrown
     *    when an invlalid collection type is supplied.
     * </p>
     */
    public void testSetJavaCollectionClassNegative() {
        Binding binding = createBinding();
        String expected = "java.lang.String";
        boolean caught = false;

        try {
            binding.setJavaCollectionClass(expected);
            assertEquals(expected, binding.getJavaCollectionClass());
        } catch (InvalidCollectionTypeException e) {
            // This indicates correct behavior
            caught = true;
        } finally {
            assertTrue(caught);
        }
    }
    
    /**
     * <p>
     *  This tests the <code>isJavaSerializable()</code> method.
     * </p>
     */
    public void testIsJavaSerializable() {
        Binding binding = createBinding();
        boolean expected = true;

        binding.setIsJavaSerializable(expected);
        assertEquals(expected, binding.isJavaSerializable());
    }

    /**
     * <p>
     *  This tests the <code>setIsJavaSerializable()</code> method.
     * </p>
     */
    public void testSetIsJavaSerializable() {
        Binding binding = createBinding();
        boolean expected1 = true;
        boolean expected2 = false;

        binding.setIsJavaSerializable(expected1);
        assertEquals(expected1, binding.isJavaSerializable());
        binding.setIsJavaSerializable(expected2);
        assertEquals(expected2, binding.isJavaSerializable());
    }
}

