/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.util;

import java.util.HashMap;
import java.util.Vector;

// JUnit imports
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

// Zeus imports
import org.enhydra.zeus.InvalidCollectionTypeException;

/**
 * <p>
 *  This is a test case for the <code>{@link CapitalizationUtils}</code> class.
 * </p>
 *
 * @author Farhat Kaleem
 * @author Brett McLaughlin
 */
public class ClassUtilsTest extends TestCase {
    
    /**
     * <p>
     *  This constructs a new <code>ClassUtilsTest</code>.
     * </p>
     *
     * @param name the name of the test case.
     */
    public ClassUtilsTest(String name) {
        super(name);
    }

    /**
     * <p>
     *  This method maeks it easier to call these
     *    tests dynamically.
     * </p>
     *
     * @return <code>TestSuite</code> - corresponds to this 
     *         <code>TestCase</code>.
     */
    public static Test suite(){
        return new TestSuite(ClassUtilsTest.class);
    }

    /**
     * <p>
     *  This tests the <code>{@link ClassUtils#isCollectionClass}</code> method.
     * </p>
     */
    public void testIsCollectionClass() {
        String [] collectionClasses =
            { "java.util.Collection", "java.util.List", "java.util.LinkedList",
              "java.util.ArrayList", "java.util.Vector",
              "java.util.Stack", "java.util.Set", "java.util.SortedSet",
              "java.util.HashSet", "java.util.TreeSet" };

        for (int i=0; i<collectionClasses.length; i++) {
            assertTrue(ClassUtils.isCollectionClass(collectionClasses[i]));
        }
    }

    /**
     * <p>
     *  This tests the <code>{@link ClassUtils#getCollectionImplClass}</code> 
     *    method.
     * </p>
     */
    public void testGetCollectionImplClass() {
        String [] collectionClasses =
            { "java.util.Collection", "java.util.List", "java.util.LinkedList",
              "java.util.ArrayList", "java.util.Vector",
              "java.util.Stack", "java.util.Set", "java.util.SortedSet",
              "java.util.HashSet", "java.util.TreeSet" };

        String [] expectedClasses =
            { "java.util.LinkedList", "java.util.LinkedList",
              "java.util.LinkedList", "java.util.ArrayList",
              "java.util.Vector", "java.util.Stack", "java.util.HashSet",
              "java.util.SortedSet", "java.util.HashSet",
              "java.util.TreeSet" };

        for (int i = 0; i < expectedClasses.length; i++) {
            assertEquals(expectedClasses[i],
                ClassUtils.getCollectionImplClass(collectionClasses[i]));
        }
    }

    /**
     * <p>
     *  This tests the <code>{@link ClassUtils#getCollectionImplClass}</code> 
     *    method.
     * </p>
     */
    public void testGetParameter() {
        Class [] paramTypes =
            { String.class, int.class, Integer.class, long.class, Long.class,
              float.class, Float.class, double.class, Double.class,
              boolean.class, Boolean.class };

        String [] values =
            { "string", "0", "0", "0", "0", "0.0f", "0.0f", "0.0d", "0.0d",
               "false", "false" };

        Object[] expected =
            { new String("string"), new Integer(0), new Integer(0),
              new Long(0), new Long(0), new Float(0.0f), new Float(0.0f),
              new Double(0.0d), new Double(0.0d), new Boolean(false),
              new Boolean(false) };

        for (int i=0; i<paramTypes.length; i++) {
            assertEquals(expected[i], 
                         ClassUtils.getParameter(values[i], paramTypes[i]));
        }
    }
    
    /**
     * <p>
     *  This tests the <code>{@link ClassUtils#getCollectionTypeAsInt}</code> 
     *    method.
     * </p>
     */
    public void testGetCollectionTypeAsInt() {
        String collectionClasses[] =
            { "java.util.List", "java.util.Array" };

        int expectedValues[] =
            { ClassUtils.COLLECTION_TYPE_LIST, 
              ClassUtils.COLLECTION_TYPE_ARRAY };

        for (int i=0; i<expectedValues.length; i++) {
            try {
                assertEquals(expectedValues[i],
                    ClassUtils.getCollectionTypeAsInt(collectionClasses[i]));
            } catch (InvalidCollectionTypeException neverHappens) {
                assertTrue(false);
            }
        }
    }
    
    /**
     * <p>
     *  This tests the <code>{@link ClassUtils#getCollectionTypeAsInt}</code> 
     *    method, in a negative fashion.
     * </p>
     */
    public void testGetCollectionTypeAsIntNegative() {
        boolean success = true;
        
        String collectionType = "java.util.List";
        try {
            int tossedOut = ClassUtils.getCollectionTypeAsInt(collectionType);
        } catch (InvalidCollectionTypeException e) {
            success = false;
        }
        assertTrue(success);
        
        collectionType = "java.util.NotACollectionType";
        success = false;
        try {
            int tossedOut = ClassUtils.getCollectionTypeAsInt(collectionType);
        } catch (InvalidCollectionTypeException e) {
            success = true;
        }
        assertTrue(success);
    }
    
    /**
     * <p>
     *  This tests the <code>{@link ClassUtils#isCollectionConstant}</code> 
     *    method.
     * </p>
     */
    public void testIsCollectionConstant() {
        int collectionConstant = ClassUtils.COLLECTION_TYPE_LIST;
        assertTrue(ClassUtils.isCollectionConstant(collectionConstant));
        
        collectionConstant = -1;
        assertTrue(!ClassUtils.isCollectionConstant(collectionConstant));
        
        collectionConstant = ClassUtils.COLLECTION_TYPE_ARRAY;
        assertTrue(ClassUtils.isCollectionConstant(collectionConstant));
        
        collectionConstant = 3078;
        assertTrue(!ClassUtils.isCollectionConstant(collectionConstant));
    }
    
    /**
     * <p>
     *  This tests the <code>{@link ClassUtils#isJavaPrimitive}</code> 
     *    method.
     * </p>
     */
    public void testIsJavaPrimitive() {
        String primitive = "Integer";
        assertTrue(!ClassUtils.isJavaPrimitive(primitive));
        
        primitive = "java.io.File";
        assertTrue(!ClassUtils.isJavaPrimitive(primitive));
        
        String primitives[] = 
            { "boolean", "byte", "char", "double", "float", "int", "long",
              "short", "String", "java.lang.String" };
              
        for (int i=0; i<primitives.length; i++) {
            assertTrue(ClassUtils.isJavaPrimitive(primitives[i]));
        }
    }
}
